// ModelSwarm.m			

#import "ModelSwarm.h"
#import <simtools.h>

@implementation ModelSwarm

+createBegin: (id)  aZone {
  ModelSwarm * obj;

  obj = [super createBegin: aZone];
  obj->numGen = 10;
  return obj;
}

-createEnd {
  [ObjectLoader load: self fromFileNamed: "model.setup"];
  return [super createEnd];
}

-buildObjects {
  Player * aPlayer;
  int pt,i;

  [randomGenerator setStateFromSeed: randomSeed];

  gen = 0;
  averagePayoff = 0.0;

  num[0] = n0;
  num[1] = n1;
  num[2] = n2;
  num[3] = n3;
  numPlayers = n0 + n1 + n2 + n3;

  popList = [List create: self];
  newList = [List create: self];
  
  for (pt = 0; pt < 4; pt++)
    for (i = 0; i < num[pt]; i++) {
      aPlayer = [Player createBegin: self];
      [aPlayer init];
      [aPlayer setPlayerType: pt];
      aPlayer = [aPlayer createEnd]; 
      [popList addLast: aPlayer];
    }

  return self;
}

-(id) getPlayers {
  return popList;
}

-(int)getNum0 {
  return num[0];
}

-(int)getNum1 {
  return num[1];
}

-(int)getNum2 {
  return num[2];
}

-(int)getNum3 {
  return num[3];
}

-(double)getAveragePayoff {
  return averagePayoff;
}

-runTournament {
  id tournament;
  id tempList;
  id player1, player2, survivor, newPlayer;
  int i, p1, p2;
  
  for (i=0; i<numPlayers; i++) {
    p1 = [uniformIntRand getIntegerWithMin: 0 withMax: numPlayers-1];
    p2 = [uniformIntRand getIntegerWithMin: 0 withMax: numPlayers-1];
    player1 = [popList atOffset: p1];
    player2 = [popList atOffset: p2];
 
    tournament = [Tournament createBegin: self];
    [tournament setPlayer1: player1 Player2: player2];
    [tournament createEnd];

    [tournament run];

    if ([uniformDblRand getDoubleWithMin: 0.0 withMax: 1.0] < selectionPressure) 
      survivor=[tournament getWinner];
    else
      survivor=[tournament getLoser];

    newPlayer=[Player create: self];
    [newPlayer init];
    [newPlayer setPlayerType: [survivor getPlayerType]];
    [newPlayer setAveragePayoff: [survivor calcAveragePayoff]];
    [newList addLast: newPlayer];
  }
  
  [tournament drop];
  [popList deleteAll];
  
  // Swapping lists
  tempList = popList;
  popList = newList;
  newList  = tempList;

  return self;
}

-reportResults {
  id index;
  id aPlayer;
  int pt;
  double payoff[4];
  double av = 0.0;

  for (pt=0; pt<4; pt++) {
    num[pt] = 0;
    payoff[pt] = 0.0;
  }

  index = [popList begin: self];
  while((aPlayer=[index next])) 
    num[[aPlayer getPlayerType]]++;  
  [index drop];

  printf("Time: %4d Num: %4d %4d %4d %4d  ", gen, num[0], num[1], num[2], num[3]);

  index = [popList begin: self];
  while((aPlayer=[index next])) {
    payoff[[aPlayer getPlayerType]]= payoff[[aPlayer getPlayerType]] + [aPlayer getAveragePayoff];
    av = av + [aPlayer getAveragePayoff];
  }  
  [index drop];

  for (pt=0; pt<4; pt++)
    if (num[pt]==0)
      printf("%6.2f ", 0.0);
    else
      printf("%6.2f ", payoff[pt]/(double)num[pt]);

  printf("\n");

  averagePayoff = av / (double)numPlayers;

  return self;
}

-step {
  if (gen==0) 
    [self reportResults];
  else {
    [self runTournament];
    [self reportResults];
  }
  gen++;
  return self;
}

@end



  


